package it4systems;

import java.awt.*;
import java.io.*;
import java.util.*;

public class MarsMap {
	public static final int EMPTY = 0;
	public static final int START = 1;
	public static final int TARGET = 2;
	public static final int STONE = 3;

	private int width;
	private int height;
	private int map[][];

	private int targetX;
	private int targetY;

	private Robot robot;
	private boolean won;
	private ArrayList startTargets;
	private ArrayList targets;

	public MarsMap(InputStream mapStream) throws Exception {
		load(mapStream);
	}

	public void load(InputStream mapStream) throws Exception {
		BufferedReader in = new BufferedReader(new InputStreamReader(mapStream));
		width = Integer.parseInt(in.readLine());
		height = Integer.parseInt(in.readLine());
		map = new int[height][width];
		startTargets = new ArrayList();
		for (int y = 0; y < height; y++) {
			String line = in.readLine();
			if (line == null) {
				break;
			}
			for (int x = 0; x < Math.min(line.length(), width); x++) {
				char c = line.charAt(x);
				switch (c) {
					case ' ' :
						map[y][x] = EMPTY;
						break;
					case 'O' :
						map[y][x] = START;
						robot = new Robot(this, 10 * x, 10 * y);
						break;
					case '+' :
						map[y][x] = TARGET;
						startTargets.add(new Point(x, y));
						break;
					default :
						map[y][x] = STONE;
						break;
				}
			}
		}
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	public int getField(int x, int y) {
		return map[y][x];
	}

	public void update(boolean left, boolean bottom, boolean right) {
		robot.update(left, bottom, right);
		robot.targetReached();
		if (isTargetReached() && robot.startReached())
			won = true;
	}

	public int getRobotX() {
		return robot.getX();
	}

	public int getRobotY() {
		return robot.getY();
	}

	public int getRobotSpeedX() {
		return robot.getSpeedX();
	}

	public int getRobotSpeedY() {
		return robot.getSpeedY();
	}

	public void reset() {
		robot.reset();
		won = false;
		targets = new ArrayList();
		targets.addAll(startTargets);
	}

	public boolean isWon() {
		return won;
	}

	public boolean isTargetReached() {
		if (targets == null)
			return false;
		return targets.size() == 0;
	}

	public boolean isTargetReached(int x, int y) {
		if (targets == null)
			return false;
		return !targets.contains(new Point(x, y));
	}

	public int getSteps() {
		return robot.getSteps();
	}

	public int getFuelUsage() {
		return robot.getFuelUsage();
	}

	public void removeTarget(int x, int y) {
		targets.remove(new Point(x, y));
	}
}
