package it4systems;

public class Robot {
	private MarsMap marsMap;
	private int startX;
	private int startY;
	private int x;
	private int y;
	private int speedX;
	private int speedY;
	private int steps;
	private int fuelUsage;

	public Robot(MarsMap marsMap, int x, int y) {
		this.marsMap = marsMap;
		this.startX = x;
		this.startY = y;
		reset();
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	public int getSpeedX() {
		return speedX;
	}

	public int getSpeedY() {
		return speedY;
	}

	private boolean hitTestImpl(int x, int y, int cellType) {
		int width = marsMap.getWidth();
		int height = marsMap.getHeight();
		if (x < 0 || x >= width * 10 || y < 0 || y >= height * 10)
			return true;
		int mx = x / 10;
		int my = y / 10;
		if (my < height && my >= 0 && mx < width && mx >= 0) {
			if (marsMap.getField(mx, my) == cellType) {
				if (cellType == MarsMap.TARGET)
					marsMap.removeTarget(mx, my);
				return true;
			}
		}
		return false;
	}

	private boolean hitTest(int x, int y, int cellType) {
		return hitTestImpl(x, y, cellType)
			|| hitTestImpl(x + 9, y, cellType)
			|| hitTestImpl(x, y + 9, cellType)
			|| hitTestImpl(x + 9, y + 9, cellType);
	}

	public void update(boolean left, boolean bottom, boolean right) {
		steps++;
		final int speedDelta = 2;
		if (left) {
			speedX -= speedDelta;
			fuelUsage++;
		}
		if (right) {
			speedX += speedDelta;
			fuelUsage++;
		}
		if (bottom) {
			speedY -= speedDelta;
			fuelUsage++;
		}
		speedY++;
		if (speedX > 10)
			speedX = 10;
		if (speedX < -10)
			speedX = -10;
		if (speedY > 10)
			speedY = 10;
		if (speedY < -10)
			speedY = -10;

		while (true) {
			int newX = x + speedX;
			int newY = y + speedY;
			if (hitTest(newX, newY, MarsMap.STONE)) {
				speedX /= 2;
				speedY /= 2;
			} else {
				x = newX;
				y = newY;
				break;
			}
			if (speedX == 0 && speedY == 0)
				break;
		}
	}

	public boolean targetReached() {
		return hitTest(x, y, MarsMap.TARGET);
	}

	public boolean startReached() {
		return hitTest(x, y, MarsMap.START);
	}

	public void reset() {
		x = startX;
		y = startY;
		speedX = 0;
		speedY = 0;
		steps = 0;
		fuelUsage = 0;
	}

	public int getSteps() {
		return steps;
	}

	public int getFuelUsage() {
		return fuelUsage;
	}
}
